# Copyright (c) 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from sahara.topology import topology_helper as t_helper

TOPOLOGY_CONFIG = t_helper.TOPOLOGY_CONFIG


def is_data_locality_enabled(**kwargs):
    return t_helper.is_data_locality_enabled()


def generate_topology_map(cluster, is_node_awareness, **kwargs):
    return t_helper.generate_topology_map(cluster, is_node_awareness)


def vm_awareness_core_config(**kwargs):
    return t_helper.vm_awareness_core_config()



# Copyright 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add is_public and is_protected flags

Revision ID: 026
Revises: 025
Create Date: 2015-06-24 12:41:52.571258

"""

# revision identifiers, used by Alembic.
revision = '026'
down_revision = '025'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('clusters',
                  sa.Column('is_public', sa.Boolean()),)
    op.add_column('cluster_templates',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('node_group_templates',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('data_sources',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('job_executions',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('jobs',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('job_binary_internal',
                  sa.Column('is_public', sa.Boolean()))
    op.add_column('job_binaries',
                  sa.Column('is_public', sa.Boolean()))

    op.add_column('clusters',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('cluster_templates',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('node_group_templates',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('data_sources',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('job_executions',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('jobs',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('job_binary_internal',
                  sa.Column('is_protected', sa.Boolean()))
    op.add_column('job_binaries',
                  sa.Column('is_protected', sa.Boolean()))



# Copyright (c) 2015 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.service.edp.oozie.workflow_creator import base_workflow
from sahara.utils import xmlutils as x


class ShellWorkflowCreator(base_workflow.OozieWorkflowCreator):

    SHELL_XMLNS = {"xmlns": "uri:oozie:shell-action:0.1"}

    def __init__(self):
        super(ShellWorkflowCreator, self).__init__('shell')

    def build_workflow_xml(self, script_name, prepare=None,
                           job_xml=None, configuration=None, env_vars=None,
                           arguments=None, files=None):
        x.add_attributes_to_element(self.doc, self.tag_name, self.SHELL_XMLNS)

        prepare = prepare or {}
        env_vars = env_vars or {}
        arguments = arguments or []
        files = files or []

        for k in sorted(prepare):
            self._add_to_prepare_element(k, prepare[k])

        self._add_configuration_elements(configuration)

        x.add_text_element_to_tag(self.doc, self.tag_name, 'exec', script_name)

        for arg in arguments:
            x.add_text_element_to_tag(self.doc, self.tag_name, 'argument', arg)

        x.add_equal_separated_dict(self.doc, self.tag_name,
                                   'env-var', env_vars)

        self._add_files_and_archives(files + [script_name], [])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_policy import policy

from sahara.common.policies import base


job_types_policies = [
    policy.DocumentedRuleDefault(
        name=base.DATA_PROCESSING_JOB_TYPES % 'get_all',
        check_str=base.UNPROTECTED,
        description='List job types.',
        operations=[{'path': '/v1.1/{project_id}/job-types',
                     'method': 'GET'},
                    {'path': '/v2/job-types',
                     'method': 'GET'}]),
]


def list_rules():
    return job_types_policies



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from oslo_utils import uuidutils
import testtools

import sahara.exceptions as ex
from sahara.service.edp.job_binaries.manila import implementation
from sahara.tests.unit import base


class _FakeShare(object):
    def __init__(self, id, share_proto='NFS'):
        self.id = id
        self.share_proto = share_proto


class TestManilaType(base.SaharaTestCase):

        def setUp(self):
            super(TestManilaType, self).setUp()
            self.manila_type = implementation.ManilaType()

        def test_validate_job_location_format(self):

            invalid_url_1 = 'manila://abc'
            invalid_url_2 = 'manila://' + uuidutils.generate_uuid()
            valid_url = 'manila://' + uuidutils.generate_uuid() + '/path'

            self.assertFalse(self.manila_type.validate_job_location_format(''))
            self.assertFalse(self.manila_type.
                             validate_job_location_format(invalid_url_1))
            self.assertFalse(self.manila_type.
                             validate_job_location_format(invalid_url_2))

            self.assertTrue(self.manila_type.
                            validate_job_location_format(valid_url))

        @mock.patch('sahara.service.edp.utils.shares.default_mount')
        @mock.patch('sahara.utils.openstack.manila.client')
        def test_copy_binary_to_cluster(self, f_manilaclient, default_mount):
            cluster_shares = [
                {'id': 'the_share_id',
                 'path': '/mnt/mymountpoint'}
            ]

            ng_shares = [
                {'id': 'the_share_id',
                 'path': '/mnt/othermountpoint'},
                {'id': '123456',
                 'path': '/mnt/themountpoint'}
            ]

            job_binary = mock.Mock()
            job_binary.url = 'manila://the_share_id/the_path'

            remote = mock.Mock()
            remote.instance.node_group.cluster.shares = cluster_shares
            remote.instance.node_group.shares = ng_shares

            info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                           remote=remote)

            self.assertItemsEqual('/mnt/mymountpoint/the_path', info)

            job_binary.url = 'manila://123456/the_path'
            info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                           remote=remote)
            self.assertItemsEqual('/mnt/themountpoint/the_path', info)

            # missing id
            default_mount.return_value = '/mnt/missing_id'

            job_binary.url = 'manila://missing_id/the_path'
            info = self.manila_type.copy_binary_to_cluster(job_binary,
                                                           remote=remote)

            self.assertItemsEqual('/mnt/missing_id/the_path', info)

        @mock.patch('sahara.utils.openstack.manila.client')
        @mock.patch('sahara.conductor.API.cluster_update')
        @mock.patch('sahara.service.edp.utils.shares.mount_shares')
        def test_prepare_cluster(self, mount_shares, cluster_update,
                                 f_manilaclient):

            cluster_shares = [
                {'id': 'the_share_id',
                 'path': '/mnt/mymountpoint'}
            ]

            ng_shares = [
                {'id': 'the_share_id',
                 'path': '/mnt/othermountpoint'},
                {'id': '123456',
                 'path': '/mnt/themountpoint'}
            ]

            job_binary = mock.Mock()

            remote = mock.Mock()
            remote.instance.node_group.cluster.shares = cluster_shares
            remote.instance.node_group.shares = ng_shares

            # This should return a default path, and should cause
            # a mount at the default location
            share = _FakeShare("missing_id")
            f_manilaclient.return_value = mock.Mock(shares=mock.Mock(
                get=mock.Mock(return_value=share)))

            job_binary.url = 'manila://missing_id/the_path'

            self.manila_type.prepare_cluster(job_binary, remote=remote)
            self.assertEqual(1, mount_shares.call_count)
            self.assertEqual(1, cluster_update.call_count)

        def test_get_raw_data(self):
            with testtools.ExpectedException(ex.NotImplementedException):
                self.manila_type.get_raw_data({})

        def test_data_validation(self):
            data = {
                "name": "test",
                "url": "man://%s" % uuidutils.generate_uuid(),
                "type": "manila",
                "description": ("incorrect url schema for")
            }
            with testtools.ExpectedException(ex.InvalidDataException):
                self.manila_type.validate(data)

            data = {
                "name": "test",
                "url": "",
                "type": "manila",
                "description": ("empty url")
            }
            with testtools.ExpectedException(ex.InvalidDataException):
                self.manila_type.validate(data)

            data = {
                "name": "test",
                "url": "manila://bob",
                "type": "manila",
                "description": ("netloc is not a uuid")
            }
            with testtools.ExpectedException(ex.InvalidDataException):
                self.manila_type.validate(data)

            data = {
                "name": "test",
                "url": "manila://%s" % uuidutils.generate_uuid(),
                "type": "manila",
                "description": ("netloc is not a uuid")
            }
            with testtools.ExpectedException(ex.InvalidDataException):
                self.manila_type.validate(data)

            data = {
                "name": "test",
                "url": "manila://%s/foo" % uuidutils.generate_uuid(),
                "type": "manila",
                "description": ("correct url")
            }
            self.manila_type.validate(data)
